;; -*- Mode: Scheme; -*-
;;; matrix routines
;;;
;;; Modified from GJS's code by BRD to work in pseudoscheme and grafix.

    
(define (cofactor m i j)
  (generate-matrix (- (matrix-ncolumns m) 1) (- (matrix-nrows m) 1)
		   (lambda (a b)
		     (matrix-ref m
				 (if (< a i)
				     a
				     (+ a 1))
				 (if (< b j)
				     b
				     (+ b 1))))))


;;; Kleanthes Konaris determinant routine, slightly edited by GJS
;;; ----------------------------------------------------------------------

;;; (iota 4) --> (0 1 2 3), as in APL.

(define (general-determinant add sub mul easy-zero?)
  (let ((zero (add)))
    (define (det m)
      (let ((cache '()))
	(define (c-det row active-column-list)
	  (if (null? (cdr active-column-list)) ;one active column
	      (matrix-ref m row (car active-column-list))
	      (let ((value (assoc (list row active-column-list) cache)))
		(if value
		    (cadr value)	; cache hit!
		    (let loop		; cache miss!
			((index 0)	
			 (remaining-columns active-column-list)
			 (answer zero))
		       (if (null? remaining-columns)
			   (begin (set! cache
					(cons (list (list row
							  active-column-list)
						    answer)
					      cache))
				  answer)
			   (let ((term
				  (matrix-ref m row (car remaining-columns))))
			     (if (easy-zero? term)
				 (loop (+ index 1)
				       (cdr remaining-columns)
				       answer)
				 (let ((contrib
					(mul term
					     (c-det (+ row 1)
						    (delete-nth index
							active-column-list)))))
				   (if (even? index)
				       (loop (+ index 1)
					     (cdr remaining-columns)
					     (add answer contrib))
				       (loop (+ index 1)
					     (cdr remaining-columns)
					     (sub answer contrib))))))))))))
	(c-det 0 (iota (vector-length m)))))
    det))

(define (delete-nth n list)
  (if (= n 0)
      (cdr list)
      (cons (car list)
	    (delete-nth (- n 1) (cdr list)))))


;;; (iota 4) --> (0 1 2 3), as in APL.

(define (iota to)
  (define (step where)
    (if (= where to)
	'()
	(cons where (step (+ where 1)))))
  (step 0))

;;; The following implements the classical adjoint formula for the
;;; inverse of a matrix.  This may be useful for symbolic applications.

(define (classical-adjoint-formula zero one add sub mul div zero?)
  (let ((det (general-determinant add sub mul zero?)))
    (define matinv
      (lambda (A)
	(let ((dim (matrix-dimension A)))
	  (if (= dim 1)
	      (vector (vector (div one (matrix-ref A 0 0))))
	      (let* ((d (det A)) (-d (sub zero d)))
		(generate-matrix dim dim
		  (lambda (j i)
		    (if (even? (+ i j))
			(div (det (cofactor A i j)) d)
			(div (det (cofactor A i j)) -d)))))))))
    matinv))

(define (matrix-dimension m)
  (let ((rows (matrix-nrows m)))
    (if (and (> rows 0)
	     (not (= rows (matrix-ncolumns m))))
	(error "Non-square matrix -- DIMENSION" m))
    rows))

