;; -*- Mode: Lisp -*-
;; brd, 2/93
; Some modifications and surgery by JAR, 3/93
;; Add graphic input capability.
;; Must load matrix.scm first.
;; run (mouse-input-demo) for a demo


;;; graphics input:

(define *timeout* 300)

(define (determine-button-click)
  (lisp:multiple-value-bind (button xraw yraw)
    (mtv:determine-button-click win:*graphics-window* '() #t #t '()
				*timeout*)
    (list button xraw yraw)))

(define wait-for-any-mouse-click determine-button-click)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; need matrix.scm

(define matrix-inverse
  (classical-adjoint-formula 0 1 + - * / zero?))


(define *cached-inverse* (cons #f #f))

(define (cached-inverse m)
  (if (eq? m (car *cached-inverse*))
      (cdr *cached-inverse*)
      (let ((i (matrix-inverse m)))
	(set! *cached-inverse* (cons m i))
	i)))


;; example usage:
;;; (get-mouse-point view (cached-inverse (view-transform view)))

(define (get-mouse-point view view-inverse)
  (let ((what (determine-button-click)))
    (let ((button (car what))
	  (xraw (cadr what))
	  (yraw (caddr what)))
      (list button
	    (windowize* (mv* view-inverse
			     (vector xraw yraw (mouse-z-value view) 1)))
	    (list xraw yraw)))))

; Hey wait a minute.  This doesn't make sense.
(define (mouse-z-value view)
  (h-z (mv* (view-transform view) (vector 0 0 0 1))))

;;; don't want integers...

(define (windowize* v)
  (let ((h (h-component v)))
    (list  (/ (x-component v) h)
	   (/ (y-component v) h))))

;;; z-fcn computes the z-value from the XY coord value
;;; draw-predicate? says when to draw the vertex.
;;; See mouse-vertex0 for an example of how to call this.

(define (mouse-vertex view view-inverse z-function draw-predicate?)
  (let ((data (get-mouse-point view view-inverse)))
    (let ((button (car data))
	  (coords (cadr data)))
      (let ((vertex (make-vertex (vector (car coords) (cadr coords)
					 (z-function coords) 1))))
	(if (draw-predicate? button vertex)
	    (begin (draw-vertex vertex view)
		   (showit)))
	(list vertex button)))))
      
;;; Usage:
;;;
;;;; (mouse-vertex0 view (cached-inverse (view-transform view)))

;;; make the vertex  with z-value = 0
;;; draw the vertex if clicked with the left or the right button.
;;;

(define (mouse-vertex0 view view-inverse)
  (mouse-vertex view view-inverse
		(lambda (coords) 0)
		(lambda (button ignore)
		  (not (middle-button? button)))))
  

(define (left-button? button) (= button 1))
(define (middle-button? button) (= button 2))
(define (right-button? button) (= button 3))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; MOUSE-CELL-0
;;; returns a vertex, edge, or face
;;; mouse using MOUSE-VERTEX0
;;;
;;; mouse a cell in and embedd it at z = 0
;;;

(define (mouse-cell-0 view view-inverse)
  (mouse-cell view view-inverse mouse-vertex0))


(define *mouse-cell-help* "
 Input new vertices with LEFT click.
 End input with RIGHT click.
 Abort input with MIDDLE click.

 MOUSE-CELL    returns a   FACE   if you select > 2 vertices.
               returns an  EDGE  if you select    2 vertices.
               returns a   VERTEX if you select   1 vertex.
 ")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;
;;; view is a view transformation
;;; view-inverse is its inverse
;;;
;;; mouse-vertex-fcn says how to mouse.
;;; for example: mouse-vertex0
;;;
;;; see mouse-cell-0 for how to call mouse-cell
;;;

;;; see *mouse-input-help* above
;;; returns a vertex, edge, or face
;;;

(define (mouse-cell view view-inverse mouse-vertex-fcn)
  (let loop ((vertices '())
	     (edges '()))
    (let ((data (mouse-vertex-fcn view view-inverse)))
      (let ((vert (car data))
	    (button (cadr data)))
	(cond ((null? button)          'timeout)
	      ((middle-button? button) 'abort)
	      (else
	       (let ((new-edges
		      (cond ((null? vertices) edges)
			    (else (let ((edge (make-edge (car vertices)
							 Vert)))
				    (draw-edge edge view)
				    (cons edge edges))))))
		 (cond ((left-button? button)
			(loop (cons Vert vertices)
			      new-edges))
		       ((right-button? button)
			(cond ((null? vertices) Vert)
			      (else
			       (let ((edge 
				      (make-edge Vert
						 (last vertices))))
				 (draw-edge edge view)
				 (showit)
				 (cond ((null? (cdr new-edges))
					edge)
				       (else (make-cell
					      (cons edge
						    new-edges))))))))))))))))


;;;;;;;;;;;;;;;;

(define (mouse-input-demo)
  (clear)
  (showit)
  (display *mouse-cell-help*)
  (let ((what (mouse-cell-0 2d-view
			    (cached-inverse (view-transform 2d-view)))))
    (cond ((member what '(abort timeout)))
	  (else 
	   (with-color :red
	     (lambda () 
	       (let ((d (dimension what)))
		 (display " dimension = ") (display d)
		 (display " Click anywhere to redraw ")
		 (wait-for-any-mouse-click)
		 (cond ((= 0 d)
			(draw-vertex what 2d-view))
		       ((= 1 d)
			(frame1 what 2d-view))
		       ((= 2 d)
			(draw-face what 2d-view))))))))
    (showit)
    what))


; Two-dimensional views...

(define 2d-view #f)

(define (set-2d-view new)
  (set! 2d-view
	(view-from-position *camera* new (h* new -1) (vector 0 1 0 1))))

(define (usual-2d-view-position view)
  (vector 0 0 (* (hnorm (view-position view)) 0.37) 1))

(set-2d-view (usual-2d-view-position view))

; Grumble.  Needs hax.scm.
(add-sync-action! (lambda ()
		    (set-2d-view (usual-2d-view-position view))))
