
; Configuration file for loading geometry software into Scheme 48.

; ,config ,load packages.scm
; ,open geometry cells constructions displaying postscript-draw ghostscript
; ,load hax.scm  -- perhaps.

; Experimental revision of module language.
(define-syntax def
  (syntax-rules (structure export)
    ((def ?name (structure ?interface ?stuff ...))
     (define-package ((?name ?interface)) ?stuff ...))
    ((def ?name (export ?spec ...))
     (define-interface ?name (export ?spec ...)))))


(def geometry-interface
  (export unitize
	  h+ h- h* h/
	  hnorm hdot hcross
	  angle-between
	  pi
	  ;; Matrix operations
	  identity
	  mm*
	  mv*
	  dilatation
	  translation
	  x-rotation
	  y-rotation
	  z-rotation
	  xy-mirror
	  yz-mirror
	  zx-mirror
	  apply-view
	  make-camera
	  view-from-position
	  view-position))

(def geometry
  (structure geometry-interface
    (open floatnum	;atan
	  scheme)
    (files geometry)))

(def cells-interface
  (export make-vertex
	  make-edge
	  make-cell
	  vertex?
	  edge?
	  face?
	  boundary
	  coboundary
	  dimension
	  negate
	  unique-id
	  complex-cells
	  for-each-edge
	  for-each-face
	  for-each-solid
	  all-vertices
	  all-faces
	  face-vertices
	  face-from-vertices
	  edge-head
	  edge-tail
	  find-edge
	  ordered-edges
	  co-ordered-edges
	  memoize
	  induced-mapping
	  invert-as-necessary
	  invert-as-necessary1
	  extend-to-complexes

	  ;; Geometric
	  centroid
	  coordinates
	  face-normal
	  for-each-visible-face
	  edge-length))

(def cells
  (structure cells-interface
    (open scheme
	  geometry
	  define-record-types
	  signals	;error
	  utils)	;reduce remq intersectq
    (files cells)))

(def constructions-interface
  (export n-gon
	  copy
	  dilate
	  prism
	  extrude
	  cone
	  dual
	  make-tetrahedron
	  make-cube
	  make-icosahedron
	  product
	  semi-truncate
	  full-truncate
	  general-truncate))

(def constructions
  (structure constructions-interface
    (open floatnum
	  scheme
	  geometry
	  cells
	  signals	;error
	  ;; formats
	  utils)	;reduce remq intersectq
    (files construct)))

(def utils-interface
  (export reduce remq intersectq
	  intersect?
	  find-if
	  last
	  last-pair
	  map-cyclically))

(def utils
  (structure utils-interface
    (open scheme)
    (files utils)))

(def displaying-interface
  (export frame frame1
	  hide  hide1
	  shade shade1
	  ;; slick
	  draw-edge
	  annotate
	  color/black color/white
	  color/red color/green color/blue
	  ))

(define-module (make-display-module draw)
  (def displaying
    (structure displaying-interface
      (open scheme
	    draw geometry cells
	    sort signals)
      (files display)))
  displaying)

; DRAW-INTERFACE specifies what things you need to implement in order
; to support a new display device.  Currently there are two implementations,
; POSTSCRIPT-DRAW and XLIB-DRAW.

(def draw-interface
  (export draw-point
	  draw-segment
	  draw-polygon
	  draw-circle
	  scrawl
	  clear
	  showit
	  with-color
	  with-line-width
	  with-gray
	  *camera*
	  initialize
	  allocate-color
	  sync
	  add-sync-action!))

(def postscript-draw
  (structure draw-interface
    (open scheme
	  geometry	;translation
	  primitives	;vm-extension, force-output
	  fluids
	  signals)
    (files postscript)))

(def xlib-draw
  (structure draw-interface
    (open scheme)
    (files xlib)))


; Mouse input (Lucid/Pseudoscheme only)

(def mouse-input
  (structure (export mouse-cell) ;etc.?
    (open scheme geometry cells)
    (files matrix
	   mouse-input)))



; Choose a particular output device.

(define-structure displaying
  (make-display-module postscript-draw)) ;or xlib-draw
