
; Postscript output routines.
; Thanks to Tim Gallivan and Kleanthes Koniaris for help with postscript
; esoterica.

; Use -Pscoot for 600 dpi printing


; 72 points per inch, page dimensions 8.5 by 11 inches

(define *width* (* 72 (+ 8 1/2)))
(define *height* (* 72 11))
(define *margin* (* 72 2))

; This is supposed to be a 35 mm camera.  1.75 cm = half of 35
; mm, and 5.0 cm is a reasonable focal length.

(define *camera*
  (make-camera 1.75 5.0
	       (- *width* *margin*) (- *height* *margin*)
	       (translation (vector (/ *width* 2) (/ *height* 2) 0 1))))

(define (draw-point x y)
  (call-error "NYI" draw-point x y))

; Draw a single edge.

(define (draw-segment x1 y1 x2 y2)
  (postscript x1 y1 "moveto" x2 y2 "lineto" "stroke"))

; Draw or fill a polygon.

(define (draw-polygon coords-list fill?)	;List of (x y)
  (let ((coords (car coords-list)))
    (postscript "newpath" (car coords) (cadr coords) "moveto"))
  (for-each (lambda (coords)
	      (postscript (car coords) (cadr coords) "lineto"))
	    (cdr coords-list))
  (postscript "closepath")
  (postscript (if fill? "fill" "stroke")))

(define (clear) (postscript "erasepage"))

(define (showit)			;Flush output buffer
  (if (fluid $postscript-interactive?)
      (begin (postscript "copypage")
	     (force-output (fluid $postscript-output-port)))))


(define (scrawl string)
  (call-error "NYI" scrawl string))

(define (draw-circle x y r)
  (call-error "NYI" draw-circle x y r))


(define (with-mumble set-mumble)
  (lambda (value thunk)
    (dynamic-wind (lambda ()
		    (postscript "gsave")
		    (set-mumble value))
		  thunk
		  (lambda ()
		    (postscript "grestore")))))

(define with-color
  (with-mumble (lambda (color)
		 (postscript (car color)
			     (cadr color)
			     (caddr color)
			     "setrgbcolor"))))

(define with-gray
  (with-mumble (lambda (gray-level)
		 (postscript gray-level "setgray"))))

(define with-line-width
  (with-mumble (lambda (width)
		 (postscript width "setlinewidth"))))

(define with-alu
  (with-mumble (lambda (alu)
		 (call-error "NYI" with-alu alu))))

(define with-line-style
  (with-mumble (lambda (style)
		 (call-error "NYI" with-line-style style))))

(define allocate-color list)


; Things to do when window size changes.

(define (initialize-window)
  ;; (win:initialize-window)
  (sync)
  )

(define *sync-actions* '())

(define (sync)
  ;; (win:sync)
  (for-each (lambda (action) (action))
	    (reverse *sync-actions*))
  (values))

(define (add-sync-action! action)
  (set! *sync-actions* (cons action *sync-actions*)))


(define $postscript-output-port
  (make-fluid (current-output-port)))

(define $postscript-interactive?
  (make-fluid #t))

(define (postscript . things)
  (for-each (lambda (thing)
	      (display #\space (fluid $postscript-output-port))
	      (display thing (fluid $postscript-output-port)))
	    things)
  (newline (fluid $postscript-output-port)))


(define (set-line-width! n)
  (postscript n "setlinewidth"))

; Eventually produce encapsulated postscript...

(define (with-postscript-to-file filename thunk)
  (call-with-output-file filename
    (lambda (port)
      (let-fluids $postscript-output-port port
		  $postscript-interactive? #f
	(lambda ()
	  (dynamic-wind (lambda ()
			  (display "%!PS-Adobe-2.0" port)
			  (newline port)
			  (display "%%EndComments" port)
			  (newline port)
			  (postscript "gsave"))
			thunk
			(lambda ()
			  (postscript "showpage" "grestore"))))))))

(define (initialize)
  (let ((port (open-output-file "/dev/null")))
    (vm-extension 98 (cons port "gs -dNOPAUSE -q -"))
    (set-fluid! $postscript-output-port port)
    (set-fluid! $postscript-interactive? #t)))

(define (terminate-ghostscript)
  (close-output-port (fluid $postscript-output-port))
  (set-fluid! $postscript-output-port (current-output-port)))

; Trouble: if you type control-C, the gs process gets killed, and a
; CLOSE on the output port proceeds to kill scheme with
;    Process scheme write on a pipe with no one to read it
; Also, since pipes get closed with fclose() instead of pclose(),
; zombies will pile up.
