
; Output routines for X window library under Pseudoscheme and Lucid
; Common Lisp.  The next layer down is provided by win.lisp, which in
; turn uses MTV... pretty baroque, I suppose.
; Not recently tested.

(lisp:require :win "win")

; Draw a little circle around a point.

(define (draw-point x y)
  (win:draw-circle x y 2))

; Draw a single edge.

(define (draw-edge edge view)
  (let ((bs (boundary edge)))
    (let ((p1 (apply-view view (coordinates (car bs))))
          (p2 (apply-view view (coordinates (cadr bs)))))
      (win:draw-segment (car p1) (cadr p1)
			(car p2) (cadr p2)))))

; Draw a face.  If fill? then color it in, otherwise make a wire
; frame.

(define (draw-face face view fill?)
  (if fill?
      (win:fill-polygon
       (map (lambda (vertex)
	      (apply-view view (coordinates vertex)))
	    (face-vertices face)))
      (for-each draw-edge (boundary face))))

; Draw a circle... I don't think anyone uses this

(define draw-circle #'win:draw-circle)

; Put some text starting at cell's centroid.

(define scrawl #'win:scrawl)

(define (clear) (win:clear 'lisp:nil))
(define showit #'win:showit)

;(define change-pen #'win:change-pen)

(define (with-mumble mumble set-mumble)
  (lambda (value thunk)
    (let ((old-value (mumble)))
      (dynamic-wind (lambda ()
		      (set-mumble value))
		    thunk
		    (lambda ()
		      (set-mumble old-value))))))

(define with-color
  (with-mumble #'win:current-color #'win:set-color))

(define with-alu
  (with-mumble #'win:current-alu
	       (lambda (alu)
		 (win:change-pen :alu alu))))

(define with-line-width
  (with-mumble #'win:current-line-width
	       (lambda (width)
		 (win:change-pen :line-width width))))

(define with-line-style
  (with-mumble #'win:current-line-style
	       (lambda (style)
		 (win:change-pen :line-style style))))

(define (with-gray intensity thunk)
  (let* ((gray-scale (force gray-scale))
	 (n (- (vector-length gray-scale) 1)))
    (with-color (vector-ref gray-scale
			    (round
			     (* intensity n)))
		(lambda ()
		  (draw-face face view)))))

; Gray scale

(define gray-scale
  (delay (make-gray-scale 33)))

(define (make-gray-scale n)
  (let ((d (/ 1.0 (- n 1))))
    (do ((i 1.0 (- i d))
	 (l '() (cons (win:allocate-color i i i) l)))
	((< i -.0001) (list->vector l)))))

(define color/black :black)


(define (initialize)
  (win:initialize-window)
  (sync))


; Things to do when window size changes.

(define *sync-actions* '())

(define (sync)
  (win:sync)
  (for-each (lambda (action) (action))
	    (reverse *sync-actions*))
  (values))

(define (add-sync-action! action)
  (set! *sync-actions* (cons action *sync-actions*)))

(add-sync-action!
  (lambda ()
    (set! *camera* (make-x-camera))))

; This is supposed to be a 35 mm camera.  1.75 cm = half of 35
; mm, and 5.0 cm is a reasonable focal length.

(define (make-x-camera)
  (make-camera 1.75
	       5.0
	       win:*width*
	       win:*height*
	       (mm* zx-mirror
		    (translation (vector (/ width 2) (/ height 2) 0 1)))))

(define *camera*
  (make-x-camera))

; These don't belong here

(define sleep #'lisp:sleep)
(define one-second 1)
